==> teamapps-ux-servlet-0.9.6-sources.jar: org/teamapps/ux/servlet/TeamAppsUxClientGate.java <==
/*-
 * ========================LICENSE_START=================================
 * TeamApps
 * ---
 * Copyright (C) 2014 - 2019 TeamApps.org
 * ---
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =========================LICENSE_END==================================
 */
package org.teamapps.ux.servlet;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.jetbrains.annotations.NotNull;
import org.teamapps.dto.UiClientInfo;
import org.teamapps.dto.UiEvent;
import org.teamapps.icons.provider.IconProvider;
import org.teamapps.server.CommandDispatcherImpl;
import org.teamapps.server.ServletRegistration;
import org.teamapps.server.SessionRecorder;
import org.teamapps.server.SessionResourceProvider;
import org.teamapps.server.UxServerContext;
import org.teamapps.webcontroller.WebController;
import org.teamapps.ux.resource.ResourceProviderServlet;
import org.teamapps.ux.resource.SystemIconResourceProvider;
import org.teamapps.ux.component.Component;
import org.teamapps.ux.component.template.BaseTemplate;
import org.teamapps.ux.session.ClientInfo;
import org.teamapps.ux.session.ClientSessionResourceProvider;
import org.teamapps.ux.session.CommandDispatcher;
import org.teamapps.ux.session.SessionContext;
import org.teamapps.ux.session.SimpleSessionContext;
import org.teamapps.uisession.QualifiedUiSessionId;
import org.teamapps.uisession.SessionClosingReason;
import org.teamapps.uisession.UiCommandExecutor;
import org.teamapps.uisession.UiSessionListener;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 *
 */
public class TeamAppsUxClientGate implements UiSessionListener {

	private final WebController webController;
	private final UiCommandExecutor commandExecutor;

	private final ObjectMapper objectMapper;
	private Map<QualifiedUiSessionId, SessionContext> sessionContextById = new ConcurrentHashMap<>();
	private Map<String, File> uploadedFilesByUuid = new ConcurrentHashMap<>();

	private final UxServerContext uxServerContext = new UxServerContext() {
		@Override
		public SessionContext getSessionContextById(QualifiedUiSessionId sessionId) {
			return sessionContextById.get(sessionId);
		}

		@Override
		public File getUploadedFileByUuid(String uuid) {
			return uploadedFilesByUuid.get(uuid);
		}
	};

	private String userSessionCommandsRecordingPath;

	public TeamAppsUxClientGate(WebController webController, UiCommandExecutor commandExecutor, ObjectMapper objectMapper) {
		this.webController = webController;
		this.commandExecutor = commandExecutor;
		this.objectMapper = objectMapper;


	}

	public void onUiSessionStarted(QualifiedUiSessionId sessionId, UiClientInfo uiClientInfo) {
		SessionRecorder sessionRecorder = null;
		if (userSessionCommandsRecordingPath != null) {
			try {
				String timeStamp = new SimpleDateFormat("yyyy.MM.dd-HH.mm.ss").format(new Date());
				File outputFile = new File(userSessionCommandsRecordingPath, "Session-" + timeStamp + ".log");
				sessionRecorder = new SessionRecorder(objectMapper, new BufferedOutputStream(new FileOutputStream(outputFile)));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		CommandDispatcher commandDispatcher = new CommandDispatcherImpl(commandExecutor, sessionId, sessionRecorder);

//		GeoIpInfo geoIpInfo = GeoIPService.getInstance().getGeoIpInfo(uiClientInfo.getIp());
//		ClientGeoIpInfo clientGeoIpInfo = new ClientGeoIpInfo(geoIpInfo.getCountryIso(),
//				geoIpInfo.getCountry(),
//				geoIpInfo.getCity(),
//				geoIpInfo.getLatitude(),
//				geoIpInfo.getLongitude());

		ClientInfo clientInfo = new ClientInfo(
				uiClientInfo.getIp(),
				null /*clientGeoIpInfo*/,
				uiClientInfo.getScreenWidth(),
				uiClientInfo.getScreenHeight(),
				uiClientInfo.getViewPortWidth(),
				uiClientInfo.getViewPortHeight(),
				uiClientInfo.getPreferredLanguageIso(),
				uiClientInfo.getHighDensityScreen(),
				uiClientInfo.getTimezoneIana(),
				uiClientInfo.getTimezoneOffsetMinutes(),
				uiClientInfo.getClientTokens(),
				uiClientInfo.getUserAgentString(),
				uiClientInfo.getClientUrl(),
				uiClientInfo.getClientParameters());
//		clientInfo.setClientSystemInfo(parseUserAgent(clientInfo.getUserAgent()));

		SessionContext context = new SimpleSessionContext(sessionId, clientInfo, commandDispatcher, uxServerContext,
				webController.getDefaultIconTheme(clientInfo.isMobileDevice()), objectMapper);
		sessionContextById.put(sessionId, context);

		context.runWithContext(() -> {
			context.setConfiguration(webController.createSessionConfiguration(context));
			context.registerTemplates(Arrays.stream(BaseTemplate.values())
					.collect(Collectors.toMap(Enum::name, BaseTemplate::getTemplate)));
			webController.onSessionStart(context);
		});
	}


//	private ClientSystemInfo parseUserAgent(String userAgentString) {
//		try {
//			Capabilities capabilities = userAgentParser.get().parse(userAgentString);
//			clientSystemInfo.setBrowser(capabilities.getBrowser());
//			clientSystemInfo.setBrowserMajorVersion(capabilities.getBrowserMajorVersion());
//			clientSystemInfo.setBrowserType(capabilities.getBrowserType());
//			clientSystemInfo.setDeviceType(capabilities.getDeviceType());
//			clientSystemInfo.setPlatform(capabilities.getPlatform());
//			clientSystemInfo.setPlatformVersion(capabilities.getPlatformVersion());
//		} catch (Exception e) {
//			e.printStackTrace();
//		}

//	}
	public Collection<ServletRegistration> getServletRegistrations() {
		ArrayList<ServletRegistration> registrations = new ArrayList<>();
		registrations.add(new ServletRegistration(new ResourceProviderServlet(createSystemIconResourceProvider(webController.getIconProvider())), "/icons/*"));
		registrations.add(new ServletRegistration(new ResourceProviderServlet(new SessionResourceProvider(sessionContextById::get)), ClientSessionResourceProvider.BASE_PATH + "*"));
		registrations.addAll(webController.getServletRegistrations(uxServerContext));
		return registrations;
	}

	@NotNull
	private SystemIconResourceProvider createSystemIconResourceProvider(IconProvider iconProvider) {
		try {
			File tempDir = File.createTempFile("temp", "temp").getParentFile();
			SystemIconResourceProvider systemIconProvider = new SystemIconResourceProvider(tempDir);
			systemIconProvider.registerStandardIconProvider(iconProvider);
			return systemIconProvider;
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}
	}

	public void onUiSessionClientRefresh(QualifiedUiSessionId sessionId, UiClientInfo clientInfo) {
		this.onUiSessionStarted(sessionId, clientInfo);
	}

	@Override
	public void onUiSessionClosed(QualifiedUiSessionId sessionId, SessionClosingReason reason) {
		SessionContext context = sessionContextById.remove(sessionId);
		if (context != null) {
			context.destroy();
		}
	}

	public void onUiEvent(QualifiedUiSessionId sessionId, UiEvent event) {
		SessionContext sessionContext = sessionContextById.get(sessionId);
		if (sessionContext != null) {
			sessionContext.runWithContext(() -> {
				sessionContext.setLastClientEventTimestamp(System.currentTimeMillis());
				String uiComponentId = event.getComponentId();
				Component component = sessionContext.getComponent(uiComponentId);
				component.handleUiEvent(event);
			});
		}
	}

	public void handleFileUpload(File file, String uuid) {
		this.uploadedFilesByUuid.put(uuid, file);
	}

	public SessionContext getSessionContextById(QualifiedUiSessionId qualifiedUiSessionId) {
		return sessionContextById.get(qualifiedUiSessionId);
	}

	public String getUserSessionCommandsRecordingPath() {
		return userSessionCommandsRecordingPath;
	}

	public void setUserSessionCommandsRecordingPath(String userSessionCommandsRecordingPath) {
		this.userSessionCommandsRecordingPath = userSessionCommandsRecordingPath;
	}
}

==> teamapps-ux-servlet-0.9.16-sources.jar: org/teamapps/ux/servlet/TeamAppsUxClientGate.java <==
/*-
 * ========================LICENSE_START=================================
 * TeamApps
 * ---
 * Copyright (C) 2014 - 2019 TeamApps.org
 * ---
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =========================LICENSE_END==================================
 */
package org.teamapps.ux.servlet;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.jetbrains.annotations.NotNull;
import org.teamapps.dto.UiClientInfo;
import org.teamapps.dto.UiEvent;
import org.teamapps.icons.provider.IconProvider;
import org.teamapps.server.CommandDispatcherImpl;
import org.teamapps.server.ServletRegistration;
import org.teamapps.server.SessionRecorder;
import org.teamapps.server.SessionResourceProvider;
import org.teamapps.server.UxServerContext;
import org.teamapps.webcontroller.WebController;
import org.teamapps.ux.resource.ResourceProviderServlet;
import org.teamapps.ux.resource.SystemIconResourceProvider;
import org.teamapps.ux.component.Component;
import org.teamapps.ux.component.template.BaseTemplate;
import org.teamapps.ux.session.ClientInfo;
import org.teamapps.ux.session.ClientSessionResourceProvider;
import org.teamapps.ux.session.CommandDispatcher;
import org.teamapps.ux.session.SessionContext;
import org.teamapps.ux.session.SimpleSessionContext;
import org.teamapps.uisession.QualifiedUiSessionId;
import org.teamapps.uisession.SessionClosingReason;
import org.teamapps.uisession.UiCommandExecutor;
import org.teamapps.uisession.UiSessionListener;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

public class TeamAppsUxClientGate implements UiSessionListener {

	private final WebController webController;
	private final UiCommandExecutor commandExecutor;

	private final ObjectMapper objectMapper;
	private Map<QualifiedUiSessionId, SessionContext> sessionContextById = new ConcurrentHashMap<>();
	private Map<String, File> uploadedFilesByUuid = new ConcurrentHashMap<>();

	private final UxServerContext uxServerContext = new UxServerContext() {
		@Override
		public SessionContext getSessionContextById(QualifiedUiSessionId sessionId) {
			return sessionContextById.get(sessionId);
		}

		@Override
		public File getUploadedFileByUuid(String uuid) {
			return uploadedFilesByUuid.get(uuid);
		}
	};

	private String userSessionCommandsRecordingPath;

	public TeamAppsUxClientGate(WebController webController, UiCommandExecutor commandExecutor, ObjectMapper objectMapper) {
		this.webController = webController;
		this.commandExecutor = commandExecutor;
		this.objectMapper = objectMapper;


	}

	public void onUiSessionStarted(QualifiedUiSessionId sessionId, UiClientInfo uiClientInfo) {
		SessionRecorder sessionRecorder = null;
		if (userSessionCommandsRecordingPath != null) {
			try {
				String timeStamp = new SimpleDateFormat("yyyy.MM.dd-HH.mm.ss").format(new Date());
				File outputFile = new File(userSessionCommandsRecordingPath, "Session-" + timeStamp + ".log");
				sessionRecorder = new SessionRecorder(objectMapper, new BufferedOutputStream(new FileOutputStream(outputFile)));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
		CommandDispatcher commandDispatcher = new CommandDispatcherImpl(commandExecutor, sessionId, sessionRecorder);

		ClientInfo clientInfo = new ClientInfo(
				uiClientInfo.getIp(),
				null /*clientGeoIpInfo*/,
				uiClientInfo.getScreenWidth(),
				uiClientInfo.getScreenHeight(),
				uiClientInfo.getViewPortWidth(),
				uiClientInfo.getViewPortHeight(),
				uiClientInfo.getPreferredLanguageIso(),
				uiClientInfo.getHighDensityScreen(),
				uiClientInfo.getTimezoneIana(),
				uiClientInfo.getTimezoneOffsetMinutes(),
				uiClientInfo.getClientTokens(),
				uiClientInfo.getUserAgentString(),
				uiClientInfo.getClientUrl(),
				uiClientInfo.getClientParameters());
//		clientInfo.setClientSystemInfo(parseUserAgent(clientInfo.getUserAgent()));

		SessionContext context = new SimpleSessionContext(sessionId, clientInfo, commandDispatcher, uxServerContext,
				webController.getDefaultIconTheme(clientInfo.isMobileDevice()), objectMapper);
		sessionContextById.put(sessionId, context);

		context.runWithContext(() -> {
			context.setConfiguration(webController.createSessionConfiguration(context));
			context.registerTemplates(Arrays.stream(BaseTemplate.values())
					.collect(Collectors.toMap(Enum::name, BaseTemplate::getTemplate)));
			webController.onSessionStart(context);
		});
	}


	public Collection<ServletRegistration> getServletRegistrations() {
		ArrayList<ServletRegistration> registrations = new ArrayList<>();
		registrations.add(new ServletRegistration(new ResourceProviderServlet(createSystemIconResourceProvider(webController.getIconProvider(), webController.getAdditionalIconProvider())), "/icons/*"));
		registrations.add(new ServletRegistration(new ResourceProviderServlet(new SessionResourceProvider(sessionContextById::get)), ClientSessionResourceProvider.BASE_PATH + "*"));
		registrations.addAll(webController.getServletRegistrations(uxServerContext));
		return registrations;
	}

	@NotNull
	private SystemIconResourceProvider createSystemIconResourceProvider(IconProvider iconProvider, List<IconProvider> customIconProvider)  {
		try {
			File tempDir = File.createTempFile("temp", "temp").getParentFile();
			SystemIconResourceProvider systemIconProvider = new SystemIconResourceProvider(tempDir);
			systemIconProvider.registerStandardIconProvider(iconProvider);
			if (customIconProvider != null) {
				customIconProvider.forEach(provider -> systemIconProvider.registerCustomIconProvider(provider));
			}
			return systemIconProvider;
		} catch (IOException e) {
			throw new IllegalStateException(e);
		}
	}

	public void onUiSessionClientRefresh(QualifiedUiSessionId sessionId, UiClientInfo clientInfo) {
		this.onUiSessionStarted(sessionId, clientInfo);
	}

	@Override
	public void onUiSessionClosed(QualifiedUiSessionId sessionId, SessionClosingReason reason) {
		SessionContext context = sessionContextById.remove(sessionId);
		if (context != null) {
			context.destroy();
		}
	}

	public void onUiEvent(QualifiedUiSessionId sessionId, UiEvent event) {
		SessionContext sessionContext = sessionContextById.get(sessionId);
		if (sessionContext != null) {
			sessionContext.runWithContext(() -> {
				sessionContext.setLastClientEventTimestamp(System.currentTimeMillis());
				String uiComponentId = event.getComponentId();
				Component component = sessionContext.getComponent(uiComponentId);
				component.handleUiEvent(event);
			});
		}
	}

	public void handleFileUpload(File file, String uuid) {
		this.uploadedFilesByUuid.put(uuid, file);
	}

	public SessionContext getSessionContextById(QualifiedUiSessionId qualifiedUiSessionId) {
		return sessionContextById.get(qualifiedUiSessionId);
	}

	public String getUserSessionCommandsRecordingPath() {
		return userSessionCommandsRecordingPath;
	}

	public void setUserSessionCommandsRecordingPath(String userSessionCommandsRecordingPath) {
		this.userSessionCommandsRecordingPath = userSessionCommandsRecordingPath;
	}
}
